﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using NHibernate.Cfg;
using NHibernate;
using DataTransfer;
using NHibernate.Expression;

namespace DataAccessLayer
{
    public class NHibernateDataProvider
    {

        #region Fields

        private ISessionFactory _sessionFactory;

        #endregion

        #region Constructors

        /// <summary>
        /// Initializes a new instance of the NHibernateDataProvider class.
        /// </summary>
        public NHibernateDataProvider()
        {
            _sessionFactory = GetSessionFactory();
        }

        #endregion

        #region Public Methods

        public int AddCustomer(Customer customer)
        {
            using (ISession session = GetSession())
            {
                using (ITransaction tx = session.BeginTransaction())
                {
                    try
                    {
                        int newId = (int)session.Save(customer);
                        session.Flush();
                        tx.Commit();
                        return newId;
                    }
                    catch (NHibernate.HibernateException)
                    {
                        tx.Rollback();
                        throw;
                    }
                }
            }
        }

        public IList<Customer> CRIT_GetCustomersByFirstname(string firstname)
        {
            using (ISession session = GetSession())
            {
                return session.CreateCriteria(typeof(Customer)).Add(new EqExpression("Firstname", firstname)).List<Customer>();
            }
        }

        public IList<Customer> CRIT_GetCustomersByFirstnameAndLastname(string firstname, string lastname)
        {
            using (ISession session = GetSession())
            {
                return session.CreateCriteria(typeof(Customer)).Add(new EqExpression("Firstname", firstname)).Add(new EqExpression("Lastname", lastname)).List<Customer>();
            }
        }

        public IList<Customer> CRIT_GetCustomersOrderedByLastnames()
        {
            using (ISession session = GetSession())
            {
                return session.CreateCriteria(typeof(Customer)).AddOrder(new NHibernate.Expression.Order("Lastname", true)).List<Customer>();
            }
        }

        public IList<Customer> CRIT_GetCutomersWithIdGreaterThan(int customerId)
        {
            using (ISession session = GetSession())
            {
                return session.CreateCriteria(typeof(Customer)).Add(new GtExpression("CustomerId", customerId)).List<Customer>();
            }
        }

        public IList<string> CRIT_GetDistinctCustomerFirstnames()
        {
            using (ISession session = GetSession())
            {
                return session.CreateCriteria(typeof(Customer)).SetProjection(Projections.Distinct(Projections.Property("Firstname"))).List<string>();
            }
        }

        public void DeleteCustomer(Customer customer)
        {
            using (ISession session = GetSession())
            {
                using (ITransaction tx = session.BeginTransaction())
                {
                    try
                    {
                        session.Delete(customer);
                        session.Flush();
                        tx.Commit();
                    }
                    catch (NHibernate.HibernateException)
                    {
                        tx.Rollback();
                        throw;
                    }
                }
            }
        }


        public Customer GetCustomerByIdAndLeaveSessionIntact(int customerId)
        {
            ISession session = GetSession();

            return session.Get<Customer>(customerId);
        }

        public Customer GetCustomerById(int customerId)
        {
            using (ISession session = GetSession())
            {
                return session.Get<Customer>(customerId);
            }

        }

        public Customer GetCustomerAndOrdersByCustomerId(int customerId)
        {
            using (ISession session = GetSession())
            {
                Customer customer = session.Get<Customer>(customerId);

                NHibernateUtil.Initialize(customer.Orders);

                return customer;
            }

        }


        public IList<CustomerFirstnameCounter> GetCustomerFirstnameCounts()
        {
            using (ISession session = GetSession())
            {
                return session.CreateQuery("select new CustomerFirstnameCounter(c.Firstname, count(c.Firstname)) from Customer c group by c.Firstname").List<CustomerFirstnameCounter>();
            }
        }

        public IList<Customer> GetCustomersByExample(Customer customerSample)
        {
            using (ISession session = GetSession())
            {
                return session.CreateCriteria(typeof(Customer)).Add(Example.Create(customerSample)).List<Customer>();
            }

        }

        public IList<Customer> GetCustomersByFirstname(string firstname)
        {
            using (ISession session = GetSession())
            {
                return session.CreateQuery("select from Customer c where c.Firstname=:fn").SetString("fn", firstname).List<Customer>();
            }
        }

        public IList<Customer> GetCustomersByFirstnameAndLastname(string firstname, string lastname)
        {
            using (ISession session = GetSession())
            {
                return session.CreateQuery("select from Customer c where c.Firstname=:fn and c.Lastname=:ln").SetString("fn", firstname).SetString("ln", lastname).List<Customer>();
            }

        }

        public IList<Customer> GetCustomersOrderedByLastnames()
        {
            using (ISession session = GetSession())
            {
                return session.CreateQuery("select from Customer c order by c.Lastname").List<Customer>();
            }
        }

        public IList<Customer> GetCustomersWithIdGreaterThan(int customerId)
        {
            using (ISession session = GetSession())
            {
                return session.CreateQuery("select from Customer c where c.CustomerId > :cid").SetInt32("cid", customerId).List<Customer>();
            }
        }

        public IList<string> GetDistinctCustomerFirstnames()
        {
            using (ISession session = GetSession())
            {
                return session.CreateQuery("select distinct c.Firstname from Customer c").List<string>();
            }
        }

        public void SaveOrUpdateCustomers(IList<Customer> customers)
        {
            using (ISession session = GetSession())
            {
                using (ITransaction tx = session.BeginTransaction())
                {
                    try
                    {
                        foreach (Customer c in customers)
                            session.SaveOrUpdate(c);
                        session.Flush();
                        tx.Commit();
                    }
                    catch (NHibernate.HibernateException)
                    {
                        tx.Rollback();
                        throw;
                    }
                }
            }

        }

        public void UpdateCustomer(Customer customer)
        {
            using (ISession session = GetSession())
            {
                using (ITransaction tx = session.BeginTransaction())
                {
                    try
                    {
                        session.Update(customer);
                        session.Flush();
                        tx.Commit();
                    }
                    catch (NHibernate.HibernateException)
                    {
                        tx.Rollback();
                        throw;
                    }
                }
            }
        }

        #endregion

        #region Private Methods

        private ISession GetSession()
        {
            return _sessionFactory.OpenSession();
        }

        private ISessionFactory GetSessionFactory()
        {
            return (new Configuration()).Configure().BuildSessionFactory();

        }

        #endregion

    }
}
